@@ -19,11 +19,13 @@
 #include <new>
 #include <stdexcept>
 
+#include "libpq-fe.h"
+
 #include "pqxx/binarystring"
 
+
 using namespace PGSTD;
 using namespace pqxx::internal;
-using namespace pqxx::internal::pq;
 
 namespace
 {
@@ -56,7 +58,7 @@ pqxx::binarystring::binarystring(const result::field &F) :
       c = p[++i];
       if (isdigit(c) && isdigit(p[i+1]) && isdigit(p[i+2]))
       {
-	c = (DV(p[c])<<9) | (DV(p[i+1])<<3) | DV(p[i+2]);
+	c = (DV(p[c])<<6) | (DV(p[i+1])<<3) | DV(p[i+2]);
 	i += 2;
       }
     }
@@ -97,14 +99,14 @@ pqxx::binarystring::const_reference pqxx::binarystring::at(size_type n) const
 
 void pqxx::binarystring::swap(binarystring &rhs)
 {
-  const size_type s(m_size);
-  // This can fail, so do it first
+  // This might fail, so do it first
   m_str.swap(rhs.m_str);
 
   // PQAlloc<>::swap() is nothrow
   super::swap(rhs);
 
   // This part very obviously can't go wrong, so do it last
+  const size_type s(m_size);
   m_size = rhs.m_size;
   rhs.m_size = s;
 }
@@ -127,11 +129,12 @@ string pqxx::escape_binary(const unsigned char bin[], size_t len)
   if (!cstr) throw bad_alloc();
   return string(cstr, escapedlen-1);
 #else
+  // TODO: Fails to escape "high" bytes!?
   string result;
   result.reserve(len);
   for (size_t i=0; i<len; ++i)
   {
-    if (bin[i] & 0x80)
+    if (bin[i] & 0x80 || bin[i] < 0x20)
     {
       char buf[8];
       sprintf(buf, "\\\\%03o", unsigned(bin[i]));
@@ -139,10 +142,6 @@ string pqxx::escape_binary(const unsigned char bin[], size_t len)
     }
     else switch (bin[i])
     {
-    case 0:
-      result += "\\\\000";
-      break;
-
     case '\'':
       result += "\\'";
       break;
