@@ -31,6 +31,8 @@
 #endif	// HAVE_UNISTD_H
 #endif	// PQXX_HAVE_SYS_SELECT_H
 
+#include "libpq-fe.h"
+
 #include "pqxx/connection_base"
 #include "pqxx/nontransaction"
 #include "pqxx/pipeline"
@@ -38,10 +40,8 @@
 #include "pqxx/transaction"
 #include "pqxx/trigger"
 
-
 using namespace PGSTD;
 using namespace pqxx::internal;
-using namespace pqxx::internal::pq;
 
 
 extern "C"
@@ -79,6 +79,12 @@ pqxx::connection_base::connection_base(const char ConnInfo[]) :
 }
 
 
+int pqxx::connection_base::backendpid() const throw ()
+{
+  return m_Conn ? PQbackendPID(m_Conn) : 0;
+}
+
+
 void pqxx::connection_base::activate()
 {
   if (!is_open())
@@ -161,6 +167,9 @@ void pqxx::connection_base::SetupState()
     throw runtime_error(Msg);
   }
 
+  for (PSMap::iterator p = m_prepared.begin(); p != m_prepared.end(); ++p)
+    p->second.registered = false;
+
   if (m_Noticer.get())
     PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, m_Noticer.get());
 
@@ -175,7 +184,12 @@ void pqxx::connection_base::SetupState()
     // in the new connection.
     auto_ptr<nontransaction> n;
     if (!m_Trans.get())
-      n=auto_ptr<nontransaction>(new nontransaction(*this, "connection_setup"));
+    {
+      // Jump through a few hoops to satisfy dear old g++ 2.95
+      auto_ptr<nontransaction>
+	nap(new nontransaction(*this, "connection_setup"));
+      n = nap;
+    }
     pipeline p(*m_Trans.get(), "restore_state");
     p.retain(m_Triggers.size() + m_Vars.size());
 
@@ -383,10 +397,12 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
     }
     else
     {
+      // Erase first; otherwise a notification for the same trigger may yet come
+      // in and wreak havoc.  Thanks Dragan Milenkovic. 
+      m_Triggers.erase(i);
+
       if (m_Conn && (R.second == ++R.first))
 	Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
-
-      m_Triggers.erase(i);
     }
   }
   catch (const exception &e)
@@ -396,6 +412,18 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
 }
 
 
+void pqxx::connection_base::consume_input() throw ()
+{
+  PQconsumeInput(m_Conn);
+}
+
+
+bool pqxx::connection_base::is_busy() const throw ()
+{
+  return PQisBusy(m_Conn);
+}
+
+
 int pqxx::connection_base::get_notifs()
 {
   int notifs = 0;
@@ -447,6 +475,34 @@ int pqxx::connection_base::get_notifs()
 }
 
 
+const char *pqxx::connection_base::dbname()
+{
+  if (!m_Conn) activate();
+  return PQdb(m_Conn);
+}
+
+
+const char *pqxx::connection_base::username()
+{
+  if (!m_Conn) activate();
+  return PQuser(m_Conn);
+}
+
+
+const char *pqxx::connection_base::hostname()
+{
+  if (!m_Conn) activate();
+  return PQhost(m_Conn);
+}
+
+
+const char *pqxx::connection_base::port()
+{
+  if (!m_Conn) activate();
+  return PQport(m_Conn);
+}
+
+
 const char *pqxx::connection_base::ErrMsg() const throw ()
 {
   return m_Conn ? PQerrorMessage(m_Conn) : "No connection to database";
@@ -478,38 +534,75 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 }
 
 
-pqxx::result pqxx::connection_base::exec_prepared(const char QueryName[],
-	int NumParams,
-	const char *const *Params,
-	int Retries)
+void pqxx::connection_base::pq_prepare(const string &name, 
+    const string &def,
+    const string &params)
+{
+  PSMap::iterator i = m_prepared.find(name);
+  if (i != m_prepared.end())
+  {
+    // Quietly accept identical redefinitions
+    if (def == i->second.definition && params == i->second.parameters) return;
+    // Reject non-identical redefinitions
+    throw logic_error("Incompatible redefinition of prepared statement "+name);
+  }
+
+  m_prepared.insert(PSMap::value_type(name, prepared_def(def, params)));
+}
+
+
+void pqxx::connection_base::unprepare(const string &name)
+{
+  PSMap::iterator i = m_prepared.find(name);
+
+  // Quietly ignore duplicated or spurious unprepare()s
+  if (i == m_prepared.end()) return;
+
+  if (i->second.registered) Exec(("DEALLOCATE " + name).c_str(), 0);
+
+  m_prepared.erase(i);
+}
+
+
+pqxx::result pqxx::connection_base::pq_exec_prepared(const string &pname,
+	int nparams,
+	const char *const *params)
 {
   activate();
-#ifdef PQXX_HAVE_PQEXECPREPARED
-  result R(PQexecPrepared(m_Conn, QueryName, NumParams, Params, NULL, NULL, 0));
 
-  while ((Retries > 0) && !R && !is_open())
+  PSMap::iterator p = m_prepared.find(pname);
+  if (p == m_prepared.end())
+    throw logic_error("Unknown prepared statement: " + pname);
+
+  // "Register" (i.e., define) prepared statement with backend on demand
+  if (!p->second.registered)
   {
-    Retries--;
-    Reset();
-    if (is_open())
-      R = PQexecPrepared(m_Conn, QueryName, NumParams, Params, NULL, NULL, 0);
+    // TODO: Wrap in nested transaction if backend supports it!
+    stringstream P;
+    P << "PREPARE " << pname << ' ' << p->second.parameters 
+      << " AS " << p->second.definition;
+    Exec(P.str().c_str(), 0);
+    p->second.registered = true;
   }
 
+#ifdef PQXX_HAVE_PQEXECPREPARED
+  result R(PQexecPrepared(m_Conn, pname.c_str(), nparams, params, 0, 0, 0));
+
   if (!R)
   {
     if (!is_open()) throw broken_connection();
     throw runtime_error(ErrMsg());
   }
-  R.CheckStatus(QueryName);
+  R.CheckStatus(pname);
   get_notifs();
   return R;
 #else
   stringstream Q;
   Q << "EXECUTE "
-    << QueryName
+    << pname
     << ' '
-    << separated_list(",", Params, Params+NumParams);
-  return Exec(Q.str().c_str(), Retries);
+    << separated_list(",", params, params+nparams);
+  return Exec(Q.str().c_str(), 0);
 #endif
 }
 
@@ -586,6 +679,11 @@ void pqxx::connection_base::InternalSetTrace() throw ()
 }
 
 
+int pqxx::connection_base::Status() const throw ()
+{
+  return PQstatus(m_Conn);
+}
+
 
 void pqxx::connection_base::RegisterTransaction(transaction_base *T)
 {
