@@ -67,6 +67,20 @@ inline void strconv(string type, const char Obj[], string expected)
   check(expected, Objstr, type);
 }
 
+const double not_a_number =
+#if defined(PQXX_HAVE_QUIET_NAN)
+  numeric_limits<double>::quiet_NaN();
+#elif defined(PQXX_HAVE_NAN)
+  nan("");
+#else
+  0.0/0.0;
+#endif
+
+struct intderef
+{
+  template<typename ITER>
+    int operator()(ITER i) const throw () { return int(*i); }
+};
 
 } // namespace
 
@@ -94,6 +108,8 @@ int main()
     testitems(I4,4);
     items<int> I5(1,2,3,4,5);
     testitems(I5,5);
+    const string l = separated_list(",",I5.begin(),I5.end(),intderef());
+    if (l != "1,2,3,4,5") throw logic_error("Separated list was '" + l + "'");
     vector<int> V2(I2);
     testitems(items<int>(V2),2);
 
@@ -127,7 +143,7 @@ int main()
     strconv("long", long_min, to_string(long_min));
     strconv("long", long_max, to_string(long_max));
     strconv("double", 0.0, "0");
-    strconv("double", numeric_limits<double>::quiet_NaN(), "nan");
+    strconv("double", not_a_number, "nan");
     strconv("string", string(), "");
     strconv("string", weirdstr, weirdstr);
 
